# Demonstrate that MATCH and MATCHALL can
# be pathologically slow when used to recursively
# match a list of whitespace separated words.
SET(fast_regex "[a-z]+( +[a-z]+)* *\r?\n")
SET(slow_regex "([a-z]+ *)+\r?\n")

# We use Jabberwocky as the input, for kicks.
# We'd actually be interested in parsing Makefiles.
FILE(READ Jabberwocky.txt stream)

# Make the input consist entirely of lines containing
# lower case words separated by blank spaces.
STRING(TOLOWER "${stream}" stream)
STRING(REGEX REPLACE "[^a-z \r\n]" "" stream "${stream}")
FILE(WRITE stream.txt "${stream}")

MACRO(TRY_SUBSTRINGS regex_string)
  STRING(LENGTH "${stream}" stream_length)
  FOREACH(count RANGE 1 ${stream_length})
    STRING(SUBSTRING "${stream}" 0 ${count} substream)
    MESSAGE("REGEX MATCH ${regex_string} on ${count} char file")
    STRING(REGEX MATCH "${${regex_string}}" out "${substream}")
  ENDFOREACH(count)
  FOREACH(count RANGE 1 ${stream_length})
    STRING(SUBSTRING "${stream}" 0 ${count} substream)
    MESSAGE("REGEX MATCHALL ${regex_string} on ${count} char file")
    STRING(REGEX MATCHALL "${${regex_string}}" out "${substream}")
  ENDFOREACH(count)
ENDMACRO(TRY_SUBSTRINGS)

TRY_SUBSTRINGS(fast_regex)
TRY_SUBSTRINGS(slow_regex)

